plugin simpleMeshMod Vu_ObjProperties
	name:"Vu_ObjProperties"
	classID:#(0x76F008E7, 0x3B2029D8)
	category:"Max Creation Graph"
(
	local pluginDefinitionFile -- the plugin definition file
	local pluginInstance -- the plugin instance created from the plugin definition
	local _meshWrapper -- the dotnet wrapper around the stack mesh pointer
	local argsType = dotNetClass "System.Object[]" -- used when creating args array
	local dotnet_ValueToDotNetObject = dotnet.ValueToDotNetObject
	local tmHandler

	-- function wrappers
	local _meshWrapper_SetValue
	local pluginInstance_UpdateMesh

	-- value wrapper local and function wrapper declarations
	local meshWrapper, meshWrapper_SetValue


	parameters main rollout:params
	(
		Visibility ui:Visibility default:1 type:#float animatable:true
		_dummy type:#boolean -- this exists to enable easy invalidation of the object        
		pluginGraph type:#filename assettype:#MaxCreationGraph readOnly:true enumAsAsset:true 
        pluginGraphDependencies type:#filenametab assettype:#MaxCreationGraph readOnly:true enumAsAsset:true tabSize:0 tabSizeVariable:true
    ) 

rollout params "Object Propertiess"
	(

		Group "Display Properties"
			(
			checkbox DisplayAsBox "Display as box"  checked:$.boxmode
			on params open do DisplayAsBox.state = $.boxmode
			on DisplayAsBox changed val do $.boxmode = not $.boxmode

			checkbox SeeThru "See Through"  checked:$.xray
			on params open do SeeThru.state = $.xray
			on SeeThru changed val do $.xray = not $.xray

			checkbox BackFaceCull "Back Face Cull"  checked:$.backfacecull
			on params open do BackFaceCull.state = $.backfacecull
			on BackFaceCull changed val do $.backfacecull = not $.backfacecull

			checkbox VertexTicks "Vertex Ticks"  checked:$.vertexTicks
			on params open do VertexTicks.state = $.vertexTicks
			on VertexTicks changed val do $.vertexTicks = not $.vertexTicks

			checkbox ShowTrajectory "Show Trajectory"  checked:$.showTrajectory
			on params open do ShowTrajectory.state = $.showTrajectory
			on ShowTrajectory changed val do $.showTrajectory = not $.showTrajectory

			checkbox ShowFrozenGray "Show Frozen in Gray"  checked:$.showFrozenInGray
			on params open do ShowFrozenGray.state = $.showFrozenInGray
			on ShowFrozenGray changed val do $.showFrozenInGray = not $.showFrozenInGray

			checkbox VertexChDisplay "Vertex Ch Display" checked:$.showVertexColors
			on params open do VertexChDisplay.state = $.showVertexColors
			on VertexChDisplay changed val do $.showVertexColors = not $.showVertexColors
			)

		Group "Rendering Control"
			(

			-- this is either ON or OFF, haven't worked out why it's not taking values between 0 - 1 yet

			--spinner Visibility "Visibility" range:[0, 1, 1] type:#float
			--on params open do $.visibility = bezier_float()
			--on Visibility changed val do $.visibility = Visibility.val

			-- workaround is to wire the Obj Vis in Obj Properties to this for now
			spinner Visibility "Visibility" range:[0, 1, 1] type:#float

			checkbox Renderable "Renderable" checked:$.renderable
			on params open do Renderable.state = $.renderable
			on Renderable changed val do $.renderable = not $.renderable

			checkbox InheritVis "Inherit Visibility" checked:$.inheritVisibility
			on params open do InheritVis.state = $.inheritVisibility
			on InheritVis changed val do $.inheritVisibility = not $.inheritVisibility

			checkbox PrimaryVis "Visible to Cameara" checked:$.primaryVisibility
			on params open do PrimaryVis.state = $.primaryVisibility
			on PrimaryVis changed val do $.primaryVisibility = not $.primaryVisibility

			checkbox SecondaryVis "Visible to Refl/Refr" checked:$.secondaryVisibility
			on params open do SecondaryVis.state = $.secondaryVisibility
			on SecondaryVis changed val do $.secondaryVisibility = not $.secondaryVisibility

			checkbox ReceiveShadows "Receive Shadows" checked:$.receiveshadows
			on params open do ReceiveShadows.state = $.receiveshadows
			on ReceiveShadows changed val do $.receiveshadows = not $.receiveshadows

			checkbox CastShadows "Cast Shadows" checked:$.castShadows
			on params open do CastShadows.state = $.castShadows
			on CastShadows changed val do $.castShadows = not $.castShadows

			checkbox ApplyAtm "Apply Atmospherics" checked:$.ApplyAtmospherics
			on params open do ApplyAtm.state = $.ApplyAtmospherics
			on ApplyAtm changed val do $.ApplyAtmospherics = not $.ApplyAtmospherics

			checkbox RenderOcc "Render Occluded Objs" checked:$.renderOccluded
			on params open do RenderOcc.state = $.renderOccluded
			on RenderOcc changed val do $.renderOccluded = not $.renderOccluded
			
			spinner GBuffID "G-Buffer ID" range:[0, 999, 0] type:#integer
			on params open do GBuffID.value = $.gbufferchannel
			on GBuffID changed val do $.gbufferchannel = GBuffID.value

			--spinner MoBlurMult "MoBlur Multiplier" range:[0, 99, 0] type:#float
			--on params open do MoBlurMult.value = $.imageMotionBlurMultiplier
			--on MoBlurMult changed val do $.imageMotionBlurMultiplier = MoBlurMult.value

			checkbox ToggleMatColour "Toggle Material Colour" checked:False
			on ToggleMatColour changed val do 	if displayColor.shaded == #material then displayColor.shaded = #object else displayColor.shaded = #material 

			)
	)

rollout inforoll "About"
	(
		Group "Vu Object Properties Info"
		(
			label lab91 "Author: Vu Nguyen" align:#left
			label lab92 "(c) Vu Nguyen 2015" align:#left
			label lab93 "Version: 1.00.00" align:#left
			label lab94 "Email: vusta@hotmail.com" align:#left
		)

	)

	fn initialize = 
	(
		pluginDefinitionFile = @"C:/Users/Honey/Autodesk/3ds Max 2016/Max Creation Graph/Tools/Vu_ObjProperties.maxtool"
		local c = dotNetClass "Viper3dsMaxBridge.ModifierPluginInstance"
		local cid = this.classId
		local refMakerWrapperClass = dotnetclass "Autodesk.Max.MaxPlus.ReferenceMaker"
		refMakerWrapper = refMakerWrapperClass._CreateWrapper this
		pluginInstance = c.CreateInstance cid[1] cid[2] pluginDefinitionFile refMakerWrapper
		pluginInstance_UpdateMesh = pluginInstance.UpdateMesh
		local meshWrapperClass = dotnetclass "Autodesk.Max.MaxPlus.Mesh"
		_meshWrapper = meshWrapperClass._CreateWrapper undefined
		_meshWrapper_SetValue = _meshWrapper._SetValue
		pluginGraph = pluginDefinitionFile
		pluginGraphDependencies = #(@"C:\Users\Honey\Autodesk\3ds Max 2016\Max Creation Graph\Compounds\ScalarToVector.maxcompound",@"C:\Program Files\Autodesk\3ds Max 2016\MaxCreationGraph\Compounds\OffsetMesh.maxcompound",@"C:\Program Files\Autodesk\3ds Max 2016\MaxCreationGraph\Compounds\DeformMeshPoints.maxcompound")

		-- value wrapper local initializations
		local meshWrapperClass = dotNetClass "Autodesk.Max.MaxPlus.Mesh"
		meshWrapper = meshWrapperClass._CreateWrapper (undefined)
		meshWrapper_SetValue = meshWrapper._SetValue

	)

	fn updateMesh = 
	(

		if (pluginInstance == undefined) then ( return undefined )
		local args = dotnet_ValueToDotNetObject #(meshWrapper, Visibility) argsType
		pluginInstance_UpdateMesh (currentTime as integer) _meshWrapper args
	)
	
	on modifyMesh do
	(
		try 
		(
			_meshWrapper_SetValue mesh
			meshWrapper_SetValue mesh

			updateMesh()
			_meshWrapper_SetValue undefined

		)
		catch()
		ok
	)

	on create do 
	(
		initialize()
	)

	on clone fromObj do 
	(
		initialize()
	)

	on load do 
	(
		initialize()
	)

	on update do 
	(
		initialize()
	)
)
